;---------------------------------------------------------------    
;irqdisp.asm - Video display routines for FINDIRQ Utility      |
;--------------------------------------------------------------|
;FINDIRQ Copyright (c) 1993                                    |
;                                                              |
;Rick Knoblaugh All rights reserved.                           |
;First Appeared in PC MAGAZINE, US Edition,                    |
;September 28, 1993                                            |
;--------------------------------------------------------------|
; 4/25/93                      Rick Knoblaugh                  |
;--------------------------------------------------------------|
;include files                                                 |
;---------------------------------------------------------------    
                include irqequ.inc
                include irqstruc.inc

code            segment public  'CODE'
                assume cs:code, ds:code, es:code

;--------------------------------------------------------------
;externals                                                   |
;--------------------------------------------------------------

                extrn   get_from_disk:near
                extrn   monoflag:byte
                extrn   monoatrib:byte


;--------------------------------------------------------------
;publics                                                     |
;--------------------------------------------------------------
                public  disp_screen, disp_pscreen
                public  cls_total, set_cursor

;--------------------------------------------------------------
;cls_total - clear the 25 line text mode screen.              |
;             Enter:                                          |
;                      bh=attribute to use                    |
;                                                             |
;--------------------------------------------------------------
cls_total	proc	near
        	xor	cx, cx		;upper left corner of entire screen
	        mov	dx, SCRN_LOWRT	;lower right

;--------------------------------------------------------------
;cls_total - clear window on the screen.                      |
;             Enter:                                          |
;                      bh=attribute to use                    |
;                      cx = upper left hand corner            |
;                      dx = lower right hand corner           |
;                                                             |
;                                                             |
;--------------------------------------------------------------
cls		proc	near
        	push	bx
	        push	si
        	push	di
	        mov     ax, BLANK_WINDOW 
                int     10h
                pop     di
                pop     si
                pop     bx
                ret
cls		endp
cls_total	endp

;--------------------------------------------------------------
;locate - locate cursor on the screen.                        |
;             Enter:                                          |
;                      dx = row/column                        |
;                                                             |
;--------------------------------------------------------------
locate		proc	near
                push    ax
        	push	bx
                push    dx
	        push	si
        	push	di
	        sub     bh, bh
        	mov     ah, LOCATE_CURSOR
                dec     dh
                dec     dl                      ;start at zero        
                int     10h
                pop     di
                pop     si
                pop     dx
                pop     bx
                pop     ax
                ret
locate		endp


;--------------------------------------------------------------
;disp_screen - Display a screen using a an offset to an array |
;              of structures containing row, col, attribute,  |
;              and string data.                               |
;                                                             |
;                                                             |
;             Enter:                                          |
;                      si = offset of data                    |
;                                                             |
;--------------------------------------------------------------
disp_screen     proc    near
                mov     bh, SCREEN_ATTRIB
                call    cls_total
disp_nocls:
	        mov     cx, 1                   ;display 1 char  
disp_s300:
                cmp     [si].str_data, STR_TERM ;last one?
                je      disp_s999
		xor     bh, bh          	;video page 0  
		mov     bl, [si].str_attrib 	;attribute to use   
                mov     dx, word ptr [si].str_col
                lea     si, [si].str_data       ;point to string
disp_s400:
		lodsb                           ;char to display  
                cmp     al, STR_TERM            ;at end?
                je      disp_s300
disp_s500:
                call    locate
	        mov     ah, WRITE_ATTR_CHAR   	;display char and attribute  
                int     10h                     ;display char and attrib
                inc     dl                      ;next column
                jmp     short disp_s400
  
disp_s999:
                ret
disp_screen     endp


;--------------------------------------------------------------
;disp_pscreen - Similar to disp_screen, except ptrs are       |
;               provided to string data versus the data       |
;               being an actual part of the display structure.|
;               Screen not cleared.                           |
;                                                             |
;               Also, if this system is monochrome, or a      |
;               switch has been set indicating that color     |
;               doesn't show up well (e.g. on LCD),           |
;               replace hard to read attributes with          |
;               black and white.                              |
;                                                             |
;             Enter:                                          |
;                      di = ptr to data                       |
;               monoatrib = TRUE if color attributes will     |
;                           not show up well on this system.  |
;                                                             |
;                                                             |
;              Exit:                                          |
;                      dl = next column for display           |
;                                                             |
;                                                             |
;  ax, bx, cx, preserved                                      |
;--------------------------------------------------------------
disp_pscreen    proc    near
                push    ax
                push    bx
                push    cx

	        mov     cx, 1                   ;display 1 char  
disp_ps300:
                cmp     [di].str_data_ptr, NULL ;last one?
                je      disp_ps999
		xor     bh, bh          	;video page 0  
		mov     bl, [di].str_attribp	;attribute to use   
                cmp     monoatrib, TRUE         ;colors hard to see?
                jne     disp_ps380
                cmp     bl, 71h                 ;blue on white?
                jne     disp_ps350
                mov     bl, 70h                 ;change to black on white
                jmp     short disp_ps380
disp_ps350:
                cmp     bl, 15                  ;white on black ?
                jne     disp_ps380
                mov     bl, 70h                 ;change to black on white
disp_ps380:

                mov     dx, word ptr [di].str_colp
                mov     si, [di].str_data_ptr   ;point to string
disp_ps400:
		lodsb                           ;char to display  
                cmp     al, STR_TERM            ;at end?
                jne     disp_ps500
                add     di, size disp_data2 
                jmp     short disp_ps300
disp_ps500:
                call    locate
	        mov     ah, WRITE_ATTR_CHAR   	;display char and attribute  
                int     10h                     ;display char and attrib
                inc     dl                      ;next column
                jmp     short disp_ps400
  
disp_ps999:
                pop     cx
                pop     bx
                pop     ax
                ret
disp_pscreen    endp

;--------------------------------------------------------------
;set_cursor - Set cursor type.                                |
;                                                             |
;          Enter:                                             |
;                 al=0	(OFF)                                 |
;                    1 (normal bottom scan -- 6/7 or 11/12)   |
;                    2 (full "insert" scan -- 0/7 or 0/12)    |
;                                                             |
;                                                             |
;  dx, cx, bx, preserved                                      |
;--------------------------------------------------------------
set_cursor      proc    near
                push    dx
                push    cx
                push    bx
        	cmp	al,2
        	jg	curs_done
        
setcurs:
        	mov	cx,2020H		;for proper EGA emulation of cursor
        					;OFF, this must be 20H
        	or	al,al
        	jz	doset			;it is turn off
        
        	mov	cl,7
                cmp     monoflag, TRUE
                jne     endscan
        	mov	cl,12			;12 is end scan for mono

endscan:
        	mov	ch,cl
        	dec	ch
        	dec	al
        	jz	doset
        	xor	ch,ch

doset:	
        	mov     ah,1                    ;set cursor type
                int     10h                     ;do video function

curs_done:
                pop     bx
                pop     cx
                pop     dx        
        	ret
set_cursor      endp




code            ends
                end               

