;---------------------------------------------------------------    
;irqsupt.asm - Supporting routines for FINDIRQ Utility         |
;--------------------------------------------------------------|
;FINDIRQ Copyright (c) 1993                                    |
;                                                              |
;Rick Knoblaugh All rights reserved.                           |
;First Appeared in PC MAGAZINE, US Edition,                    |
;September 28, 1993                                            |
;--------------------------------------------------------------|
; 4/25/93                      Rick Knoblaugh                  |
;--------------------------------------------------------------|
;include files                                                 |
;---------------------------------------------------------------    
                include irqequ.inc
                include irqstruc.inc

code            segment public  'CODE'
                assume cs:code, ds:code, es:code
;--------------------------------------------------------------
;Eexternals                                                   |
;--------------------------------------------------------------
                extrn   data_path:byte          ;file spec for data file
                extrn   pic_mask:word           ;master and slave PIC mask values

                extrn   data_file:byte, installf:byte
                extrn   atclassf:byte                

                extrn   pc_std_dev:word, at_std_dev:word
                extrn   pc_basic_dev:word, at_basic_dev:word
                extrn   com3_desc:byte, com4_desc:byte

                extrn   other_desc:byte, mouse_desc:byte
                extrn   device_desc:byte
                                     
                extrn   int_tab_off:word, int_tab_seg:word
                extrn   FILE_NAME_LEN:abs
        
                extrn   num_com:byte, num_prt:byte
                
                extrn   dos_handler_seg:word
                extrn   drv_ptr:dword
                extrn   block_desc:byte, block_abbrv:byte
                extrn   drv_desc_ptr:word
                extrn   ver_irq_mask:word
                extrn   monoflag:byte
                extrn   monoatrib:byte
                extrn   dos_ver:word
                extrn   math_present:byte

          
;--------------------------------------------------------------
;Publics                                                      |
;--------------------------------------------------------------
                public  put_to_disk, get_from_disk 
                public  get_switch, get_set_int, get_path
                public  find_devices, find_com_lpt, set_desc_entry
                public  find_drivers, see_if_install
                public  get_desc_entry, load_desc   
                public  ck_for_dos, is_it_rom
                public  verify_enables, ck_monitor, get_stacks_seg

;--------------------------------------------------------------
;put_to_disk  - Write new PIC mask values to data file.       |
;                                                             |
;          Enter:                                             |
;                  ds=local data segment                      |
;           data_path=file spec for data file                 |
;            pic_mask=master and slave PIC mask values        |
;                                                             |
;           Exit:                                             |
;                                                             |
;                 cf=set if problem writing file              |
;                                                             |
;--------------------------------------------------------------
put_to_disk     proc    near
                mov     dx, offset data_path
                xor     cx, cx                  ;normal file attributes
                mov     ah, DOS_CREATE_FILE
                int     21h
                jc      put_to_d999             ;if failure, exit     
                mov     bx, ax                  ;file handle
                mov     dx, offset pic_mask
                mov     cx, 2                   ;write two bytes
                mov     ah, DOS_WRITE_FILE 
                int     21h
                mov     ah, DOS_CLOSE_FILE 
                int     21h
put_to_d999:
                ret
put_to_disk     endp        


;--------------------------------------------------------------
;get_from_disk  - Read PIC mask value from data file.         |
;                                                             |
;          Enter:                                             |
;                                                             |
;                  ds=local data segment                      |
;           data_path=file spec for data file                 |
;            pic_mask=master and slave PIC mask values        |
;                                                             |
;           Exit:                                             |
;                                                             |
;                 cf=set if problem reading file              |
;                                                             |
;--------------------------------------------------------------
get_from_disk   proc    near
                mov     dx, offset data_path
                xor     al, al                  ;open for reading
                mov     ah, DOS_OPEN_FILE
                int     21h
                jc      get_from_d999
                mov     bx, ax                  ;file handle
                mov     dx, offset pic_mask
                mov     cx, 2                   ;read two bytes
                mov     ah, DOS_READ_FILE 
                int     21h
                jc      get_from_d999
                mov     ah, DOS_CLOSE_FILE 
                int     21h
get_from_d999:
                ret
get_from_disk   endp


;--------------------------------------------------------------
;get_switch - look for a switch character (and something      |
;             just after the switch)                          |
;                                                             |
;             Enter:                                          |
;                      cx=number of bytes to search           |
;                   ds:si=ptr to string to search             |
;                                                             |
;             Exit:                                           |
;                      cx=bytes remaining after switch        |
;                      cf=set if invalid                      |
;                      else                                   |
;                      cf=clear                               |
;                      si=position just after switch          |
;                      al=value forced to uppercase           |
;                                                             |
;                                                             |
;--------------------------------------------------------------
get_switch      proc    near
                jcxz    get_swit_800
                call    skip_white_sp
                jcxz    get_swit_800
                lodsb                           ;get byte
                cmp     al, SWITCH_CHAR         ;is it a switch char?
                jne     get_swit_900
                dec     cx                      ;count the switch
                jcxz    get_swit_900
                lodsb                           ;get byte
                cmp     al, 'a'                 ;lowercase?
                jb      get_swit_800
                and     al, 0dfh                ;if so, force to upper
get_swit_800:     
                clc                  
                jmp     short get_swit_999
get_swit_900:     
                stc       
get_swit_999:     
                ret
get_switch      endp

skip_white_sp   proc    near
                cmp     byte ptr [si], ' '
                je      skip_w200
                cmp     byte ptr [si], TAB
                jne     skip_w900

skip_w200:
                inc     si
                loop    skip_white_sp
skip_w900:
                ret
skip_white_sp   endp



;--------------------------------------------------------------
;get_set_int - For a given interrupt vector, store contents   |
;              and load with new isr address.                 |
;                                                             |
;             Enter:                                          |
;                                                             |
;             al = int number                                 |
;             di = offset of location to store contents       |
;             dx = offset new isr                             |
;                  (segments for these assumed to be same)    |
;                                                             |
;                                                             |
;    es not preserved.                                        |
;--------------------------------------------------------------
get_set_int     proc    near
                mov     ah, DOS_GET_INT         
                int     21h                     ;es:bx returned
                mov     [di].d_offset, bx       ;save offset 
                mov     bx, es
                mov     [di].d_segment, bx      ;save segment
                
                mov     ah, DOS_SET_INT         
                int     21h                     ;ds:dx has our routine
                ret
get_set_int     endp        


;--------------------------------------------------------------
;get_path - Go pull path from which FINDIRQ was invoked out   |
;           of the environment.  Place this path in front of  |
;           the data filename.                                |
;                                                             |
;             Enter:                                          |
;                      ds=psp segment                         |
;             Exit:                                           |
;                data_path = ASCIIZ string containing path    |
;                            and file name.                   |
;                                                             |
;    Only ds preserved.                                       |
;--------------------------------------------------------------
get_path        proc    near
                push    ds
                mov     di, offset data_path                

                mov     bx, psp_environ_seg
                mov     bx, [bx]                ;get environment segment
                mov     es, bx                  ;es=environment segment
                call    retrieve_path           ;pull path out of environment
                                                   
                assume  ds:nothing
                push    es
                pop     ds                      ;get ds=environment segment

                push    cs
                pop     es                      ;get es=local data segment
                jc      get_path500             ;only if not found
get_path100:
                cmp     si, bx                  ;at end of path?
                je      get_path500
                movsb
                jmp     short get_path100       ;if not, keep moving

get_path500:
                pop     ds                      ;restore local data segment
                assume  ds:code                 ;COM file (psp same seg)
                mov     si, offset data_file    ;file name
                mov     cx, FILE_NAME_LEN       ;its length
                rep     movsb
                ret
get_path        endp


;--------------------------------------------------------------
;see_if_install - Using the multiplex interrupt, issue a      |
;                 get installed state function to our handler |
;                 id.  If it returns present, also issue      |
;                 the next higher function to retrieve the    |
;                 segment and offset of the int table.        |
;                                                             |
;                                                             |
;             Enter:                                          |
;                      ds=our local data segment              |
;                                                             |
;             Exit:                                           |
;                installf=TRUE if resident portion found      |
;                         else FALSE                          |
;                      cf=set if resident                     |
;                                                             |
;    ax, cx, ds preserved.                                    |
;--------------------------------------------------------------
see_if_install  proc    near 
                push    ax
                push    cx
                push    ds
                mov     installf, FALSE         ;default to not installed
                mov     ax, ((FINDIRQ_ID shl 8) OR GET_INST_STATE) ;our id
                int     INSTALL_INT  
                or      al, al                  ;installed?
                jz      see_if_in999            ;if not, exit
                mov     ax, ((FINDIRQ_ID shl 8) OR GET_INST_STATE + 1) ;our id
                mov     cx, 'Q!'                 
                mov     bx, offset installf     ;ds:bx point to our flag
                int     INSTALL_INT  
                mov     cs:int_tab_off, dx
                mov     cs:int_tab_seg, ax      ;&int table
                mov     al, installf
                shr     al, 1                   ;put into carry flag
see_if_in999:

                pop     ds
                pop     cx
                pop     ax
                ret
see_if_install  endp         


        
;--------------------------------------------------------------
;find_devices - Know about standard devices.  Now look for    |
;               other devices.  If found, set the IRQ for     |
;               the device as enabled in our copies of the    |
;               PIC masks.  Store offset of description       |
;               of device in array.  There may be more than   |
;               one device using a given IRQ.                 |
;                                                             |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                atclassf = TRUE if AT class machine          |
;                                                             |
;                pic_mask holds PIC mask values               |
;                                                             |
;              Exit:                                          |
;                                                             |
;                 pic_mask updated                            |
;                                                             |
;                                                             |
;--------------------------------------------------------------
find_devices    proc    near

                call    find_com_lpt            

                sub     cx, cx                  ;default to no mouse
                mov     ax, GET_MOUSE_TYPE
                int     33h                     ;mouse int
                jcxz    find_dev400             ;jmp if no mouse 
                sub     bh, bh
                cmp     ch, 4                   ;PS/2 Mouse?
                jne     find_dev100
                mov     bl, 12                  ;if so, uses irq12
                and     pic_mask.d_slave, not (mask atirq12) ;ensure we know
                jmp     short find_dev200

find_dev100:
                mov     bl, cl
                mov     al, 1
                shl     al, cl                  ;get mask position
                not     al
                and     pic_mask.d_master, al   ;ensure we know
find_dev200:
;
;Go find an empty slot for this IRQ's descriptions and put mouse 
;on the list of device descriptions for this IRQ
;
;
                mov     dx, offset mouse_desc 
                mov     ax, bx                  ;pass IRQ number
                call    set_desc_entry          ;find slot for

find_dev400:
                ret
find_devices    endp        
 
 
;--------------------------------------------------------------
;find_com_lpt - Based on int 11h information, update retrieve |
;               and store descriptions for the COM and LPT    |
;               devices for the appropriate IRQs.  In the     |
;               future, may want to change this code so that  |
;               COM ports and LPTs are dynamically detected   |
;               (i.e. actually take over all hardware         |
;               interrupts, enable all IRQs, talk directly    |
;               to the device I/O space and see where it      |
;               responds) - this would find "steered" IRQs.   |
;                                                             |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                                                             |
;              Exit:                                          |
;                      descriptions updated.                  |
;                                                             |
;--------------------------------------------------------------
find_com_lpt    proc    near
                xor     al, al                  ;IRQ counter
                mov     ah, pic_mask.d_master
                mov     si, offset at_std_dev
                cmp     atclassf, TRUE          ;AT machine?
                je      find_com_030
                mov     si, offset pc_std_dev
find_com_030:
                mov     cx, 8                   ;8 IRQs on master PIC
find_com_050:
                sub     bh, bh
                mov     bl, al                  ;irq number
                shl     bx, 1                   ;index into words
find_com_100:
                mov     dx, [si + bx]           ;get ptr to description
                shr     ah, 1                   ;get flag indicating active
                jc      find_com_600            ;jmp if not active
find_com_200:
                cmp     al,  3                  ;COM2 irq
                jne     find_com_250           ;if not
                cmp     num_com, 2
                jnb     find_com_500           ;ok, it's COM2
                jmp     short find_com_600
find_com_250:
                cmp     al, 4                   ;COM1 irq
                jne     find_com_300
                cmp     num_com, 1
                jnb     find_com_500           ;ok, it's COM1
                jmp     short find_com_600

find_com_300:
                cmp     al, 7                   ;LPT1 irq
                jne     find_com_350
                cmp     num_prt, 1
                jnb     find_com_500           ;ok, it's LPT1:
                jmp     short find_com_600
find_com_350:
                cmp     atclassf, TRUE
                jne     find_com_600           ;if PC class

                cmp     al, 5                   ;LPT2 irq on AT
                jne     find_com_600
                cmp     num_prt, 2
                jnb     find_com_500           ;ok, it's LPT2:

                jmp     short find_com_600
;                   
;Check to see if there is a description for the device
;
find_com_500:
                or      dx, dx                  ;is there a description?
                jz      find_com_600           ;if not, try next IRQ
;
;When change to dynamic detection will want to show enable in our
;pic mask here also
;
;
;
                call    set_desc_entry
find_com_600:
                inc     al                      ;next IRQ
                loop    find_com_050

;Look for com3 and com4
;
                cmp     num_com, 2              ;two or less?
                jbe     find_com_800
                mov     al, 4                   ;IRQ4 is used for com3
                mov     dx, offset com3_desc 
                call    set_desc_entry
                cmp     num_com, 4              ;< 4 com ports?
                jb      find_com_800
                mov     al, 3                   ;IRQ3 is used for com4
                mov     dx, offset com4_desc 
                call    set_desc_entry

find_com_800:

                ret
find_com_lpt    endp

;--------------------------------------------------------------
;set_desc_entry - Add description to an array of              |
;                 offsets of strings containing descriptions  |
;                 of devices assigned to a particular IRQ.    |
;                                                             |
;                 Note: each IRQ can have a number of         |
;                       descriptions.  If it has maxed out    |
;                       on descriptions, nothing is added.    |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                      al = IRQ number for which description  |
;                           is desired.                       |
;                      dx = offset of description to add.     |
;                                                             |
;              Exit:                                          |
;                                                             |
;                      di = base of array of offsets          |
;                                                             |
;   cx preserved.                                             |
;--------------------------------------------------------------
set_desc_entry  proc    near
                push    cx
                call    get_desc_entry
                mov     cx, (size device_desc_off) / DESC_ENTRY ;number of descriptions

set_desc500:
                cmp     [di].descrip, 0         ;this one available?
                je      set_desc800
                add     di, DESC_ENTRY          ;if not, try next one
                loop    set_desc500                     
                jmp     short set_desc900       ;oh well, too many   

set_desc800:
                mov     [di].descrip, dx        ;move in the description        
set_desc900:

                pop     cx
                ret

set_desc_entry  endp


;--------------------------------------------------------------
;get_desc_entry - Return offset of the base of an array of    |
;                 offsets of strings containing descriptions  |
;                 of devices assigned to a particular IRQ.    |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                      al = IRQ number for which description  |
;                           is desired.                       |
;              Exit:                                          |
;                                                             |
;                      di = base of array of offsets to       |
;                           descriptions for the IRQ          |
;   ax, cx preserved.                                         |
;--------------------------------------------------------------
get_desc_entry  proc    near
                push    ax
                push    cx
                mov     di, offset device_desc         ;get base of descriptions
                mov     cx, size device_desc_off
                mul     cl                      ;offset for this IRQ
                add     di, ax                  ;add it in 
                pop     cx
                pop     ax
                ret
get_desc_entry  endp





load_desc       proc    near
                push    bp
                xor     al, al                  ;IRQ counter
                mov     bp, pic_mask            ;our images of slave/master
                mov     cx, 16                  ;16 IRQs on AT
                mov     si, offset at_basic_dev
                cmp     atclassf, TRUE          ;AT machine?
                je      load_desc_050
                mov     si, offset pc_basic_dev
                mov     cx, 8                   ;only 8 IRQs on PC
load_desc_050:

                sub     bh, bh
                mov     bl, al                  ;irq number
                shl     bx, 1                   ;index into words

load_desc_100:
                mov     dx, [si + bx]           ;get ptr to description
                shr     bp, 1                   ;get flag indicating active
                jnc     load_desc_200           ;jmp if active
                cmp     atclassf, TRUE          ;AT machine?
                jne     load_desc_150
                cmp     al, 2                   ;if AT and IRQ2 and not active
                je      load_desc_200           ;go assign "cascade"
load_desc_150:
                xor     dx, dx                  ;no description - not active

load_desc_200:
;                   
;Check to see if there is a description for the device
;
                or      dx, dx                  ;is there a description?
                jz      load_desc_600           ;if not, try next IRQ
                call    set_desc_entry
load_desc_600:
                inc     al                      ;next IRQ
                loop    load_desc_050
load_desc_999:
                pop     bp
                ret
load_desc       endp


;--------------------------------------------------------------
;find_drivers - For all IRQs that are active, look to see if  |
;               we have a description for the device.  If we  |
;               don't, the device using this IRQ must be      |
;               something we don't know about, such as a      |
;               sound board, SCSI adapter, etc.  In this case,|
;               go get the driver name or program name and    |
;               put it in the description for the IRQ.        |
;                                                             |
;               In the event that a driver or TSR (with       |
;               PSP) is not found, assign "Other" for the     |
;               device description.                           |
;                                                             |
;                                                             |
;          Enter:                                             |
;                  ds=local data segment                      |
;                atclassf = TRUE if AT class machine          |
;                                                             |
;                pic_mask = PIC mask values.                  |
;                                                             |
;          device_desc = offset to array of description       |
;                        offsets.                             |
;                                                             |
;           Exit:                                             |
;                                                             |
;  ds, bp preserved, es trashed.                              |
;--------------------------------------------------------------
find_drivers    proc    near
                push    bp
                mov     ah, DOS_LIST_LISTS
                int     21h                     ;get es:bx ptr to lists
                
                cmp     dos_ver, 0003h          ;DOS 3.0?
                jne     find_drv_050
                lea     bx, es:[bx].nul_dev_head30 ;get ptr to NUL device 
                jmp     short find_Drv_080
find_drv_050:
                lea     bx, es:[bx].nul_dev_head   ;get ptr to NUL device 
find_drv_080:
                mov     ax, es
                mov     drv_ptr.d_segment, ax   ;save ptr to NUL device (start
                mov     drv_ptr.d_offset, bx    ;of driver chain)

                xor     al, al                  ;IRQ counter
                mov     bp, pic_mask
                mov     cx, 16                  ;16 IRQs on AT
                cmp     atclassf, TRUE          ;AT machine?
                je      find_drv_100
                shr     cx, 1                   ;only 8 IRQs on PC
find_drv_100:
                shr     bp, 1                   ;check IRQ
                jc      find_drv_600            ;skip if not active
find_drv_110:
                call    get_desc_entry          ;get description for IRQ
                xor     bx, bx
                cmp     [di], bx                ;is there one?
                je      find_drv_125
;
;If there is already a description for the device, don't need to look for
;driver.  However, there is one exception.  If we are on an AT, we always
;assign an initial description of "cascade" to IRQ2.  If that is the only
;description we have (e.g. we haven't got a mouse on IRQ2, etc.), go ahead and
;look for a driver using IRQ2.
;

                cmp     atclassf, TRUE          ;AT machine?
                jne     find_drv_600            ;if not, no cascade
                cmp     al, 2                   ;doing IRQ2?
                jne     find_drv_600            ;if not, no test
                cmp     [di + 2], bx            ;check 2nd description (bx=0)
                jne     find_drv_600
                add     di, 2                   ;next description entry

find_drv_125:
                mov     es, bx                  ;point to vector

                mov     bl, al                  ;irq number
                cmp     al, 8
                jb      find_drv_150
                add     bl, (BEG_SLAVE_INTS  - 8)
                jmp     short find_drv_175

find_drv_150:
                add     bl, BEG_INTS       
find_drv_175:
                shl     bx, 1
                shl     bx, 1                   ;get offset into vectors
;See if pointing to DOS stack handlers.  If so, carry is not set and es:bx
;points to address of original ISR.
;
                call    ck_for_dos
                jnc     find_drv_200            ;es:bx point to ISR

                les     bx, es:[bx]             ;point to ISR

find_drv_200:
                call    load_drv_desc  
find_drv_600:
                inc     al                      ;advance IRQ counter
                loop    find_drv_100
find_drv_900:          
                pop     bp
                ret
find_drivers    endp




;--------------------------------------------------------------
;load_drv_desc - Retrieve and load driver description.        |
;                                                             |
;          Enter:                                             |
;                  ds=local data segment                      |
;                  di=offset of entry for description         |
;               es:bx=ptr to ISR for which description is     |
;                     needed.                                 |
;        device_desc = offset to array of description         |
;                      offsets.                               |
;       drv_desc_ptr = offset of next work description area.  |
;                                                             |
;           Exit:                                             |
;                 [di]=offset of description.                 |
;         drv_desc_ptr advanced to next work description area.|
;                                                             |
;  all registers saved.                                       |
;--------------------------------------------------------------
load_drv_desc   proc    near
                push    ax                      ;save picmask/IRQ counter
                push    cx                      ;save loop count
                push    di                      ;save offset of description entry

                push    bx                      ;save offset
                mov     dx, es                  ;pass address in dx:bx        
                call    is_it_driver            ;see if driver or TSR
                pop     dx                      ;get back actual offset (was bx)
                jnc     load_drv_300            ;jmp if not a driver
                mov     dx, offset block_desc   ;"Block Device"
                cmp     al, 2                   ;IRQ2? if so, may have
                jne     load_drv_290            ;"Cascade" if AT machine - use
                mov     dx, offset block_abbrv  ;abbreviated description to fit

load_drv_290:
                test    es:[bx].dev_attrib, 8000h ;char driver?
                jz      load_drv_550            ;if not, go use "Block Device"
                call    mov_char_name           ;if so, move char device name
                jmp     short load_drv_450
load_drv_300:
;
;Wasn't a driver.  Must be a TSR.  Using the pointer to the environment
;from the TSR's PSP, find the name of executable.
;
;
                mov     bx, dx                  ;offset of ISR
                call    determine_psp           ;return nc if PSP found
;
;If a PSP is found, carry will NOT be set.  Also, if PSP is found, and
;ax is nonzero, ax holds the segment of the environment.  If ax is zero
;then the environment was freed.  If such cases, es:bx will be returned
;as a pointer to the filename description in the MCB (if DOS is 4.x or 
;better - otherwise bx=0)
;
                jnc     load_drv_350
load_drv_330:
                mov     dx, offset other_desc   ;else can't get filename
                jmp     short load_drv_550      ;so, go assign "Other"
load_drv_350:
                or      ax, ax                  ;environment found?
                jnz     load_drv_370            ;if so, go get filename
                or      bx, bx                  ;name in MCB? (DOS 4.x or >)
                jz      load_drv_330            ;if not, print "Other"
                jmp     short load_drv_400      ;go get name
load_drv_370:
                mov     es, ax
                call    retrieve_path           ;get ptr to filename
                jc      load_drv_330            ;if data not right
load_drv_400:
                mov     di, drv_desc_ptr 
                mov     dx, di                  ;save description offset
                call    mov_file_name
load_drv_450:
                mov     al, STR_TERM
                stosb                           ;terminate it
                add     drv_desc_ptr, size drv_descrip
load_drv_550:
                pop     di
                pop     cx
                pop     ax
                mov     [di], dx                ;store offset of driver/program name
                ret
load_drv_desc   endp



;--------------------------------------------------------------
;determine_psp - An ISR was found and it is not part of a     |
;                device driver.  Search the MCB chain for     |
;                PSP associated with code that would contain  |
;                the ISR.  If found, check the environment    |
;                entry for the PSP.  If environment has not   |
;                been freed, return ax=environment segment. If|
;                not found and DOS is 4.x or better, return   |
;                es:bx as ptr to filename in MCB.             |
;                                                             |
;                                                             |
;          Enter:                                             |
;                  es:bx = ptr to ISR                         |
;                                                             |
;           Exit:                                             |
;                     cy set if PSP NOT found else nc         |
;                     also if found ax=environment seg (zero  |
;                                      if it was freed)       |
;                                                             |
;                          if found and ax=0, es:bx=ptr       |
;                                       to filename in MCB    |
;                                       (zero if < DOS 4)     |
;                                                             |
;   ds preserved.                                             |
;--------------------------------------------------------------
determine_psp   proc    near
                push    ds
                mov     dx, es                  ;segment of ISR
                mov     cl, 4
                shr     bx, cl                  ;convert offset to paragraphs
                add     dx, bx                  ;paragraph location of code

                mov     ah, DOS_LIST_LISTS
                int     21h
                mov     di, es:[bx - 2]         ;segment of first MCB
                mov     es, di
                xor     bx, bx                  ;offset zero in MCB
                mov     si, es                  ;save MCB segment
determine_p200:
                inc     si                      ;ready to advance to next one

                cmp     es:[bx].mcb_sig, 'Z'    ;at end of chain?
                je      determine_p900          ;if so, not found
                cmp     es:[bx].mcb_sig, 'M'    ;valid entry?
                jne     determine_p900          ;if not, MCBs not right

determine_p300:
                mov     cx, es:[bx].mcb_owner   ;get MCB owner segment
                mov     di, cx                  ;save it
                mov     ax, es:[bx].mcb_size    ;and its size
                cmp     cx, si                  ;is this a PSP?
                jne     determine_p700          ;if not, go advance to next PSP

                cmp     dx, cx                  ;compare segment
                jb      determine_p700          ;beneath the segment
                add     cx, ax                  ;get end of the area
                cmp     dx, cx
                ja      determine_p700          ;above the segment

                assume  ds:nothing
                mov     ds, di                  ;ds=seg of owner
                mov     ax, [bx].psp_environ_seg

                or      ax, ax                  ;freed the environment?
                jz      determine_p400          ;if so, try plan b

                dec     ax                      ;1 paragraph back from "environment"
                mov     ds, ax                  ;is segment of MCB for it
                inc     ax                      ;back to "environment"
                cmp     [bx].mcb_owner, di      ;belongs to PSP?
                je      determine_p999          ;yes, exit nc and ax=segment

determine_p400:
                xor     ax, ax                  ;indicate no environment found
                cmp     byte ptr cs:dos_ver, 4  ;DOS 4 or better?
;
;if yes, we can return filename     
;if not, bx=0 indicating must print "Other"
;
                cmc                             ;make carry clear if < DOS 4
                jnb     determine_p999          ;exit nc and bx=0 if < DOS 4
                                                
determine_p500:
                clc
                lea     bx, es:[bx].mcb_fname 
                jmp     short determine_p999    ;exit nc and es:bx ptr to filename

determine_p700:
                add     si, ax                  ;get to next MCB
                mov     es, si
                jmp     short determine_p200

determine_p900:
                stc
determine_p999:
                assume  ds:code
                pop     ds
                ret
determine_psp   endp


mov_char_name   proc    near
                push    ds

                mov     dx, drv_desc_ptr 
                push    es
                pop     ds
                assume  ds:nothing
                lea     si, [bx].dev_char_name
                mov     cx, CHAR_NAME_LEN
                push    cs
                pop     es
                mov     di, dx
                rep     movsb                   ;move char name to our storage
                assume  ds:code

                pop     ds
                ret
mov_char_name   endp




;--------------------------------------------------------------
;mov_file_name - Moves the filename to description area.      |
;                This routine can be called when es:bx        |
;                points to a filename found in an environment |
;                block (filename has extention) or es:bx can  |
;                point to filename found in an MCB (only      |
;                null terminated main filename portion        |
;                provided).                                   |
;                                                             |
;          Enter:                                             |
;                  es:bx = ptr to filename                    |
;                     di = offset of description destination  |
;           Exit:                                             |
;   ds, cx preserved.                                         |
;--------------------------------------------------------------
mov_file_name   proc    near
                push    cx
                push    ds
                mov     cx, 8                   ;max filename length
                mov     si, bx                  ;point to filename

                push    es
                pop     ds
                assume  ds:nothing
                push    cs
                pop     es

mov_file_100:
                movsb                           ;move filename to our storage
                cmp     byte ptr [si], '.'      ;did whole thing?
                je      mov_file_999
                cmp     byte ptr [si], NULL     ;did whole thing?
                loopne  mov_file_100
                assume  ds:code
mov_file_999:
                pop     ds
                pop     cx
                ret
mov_file_name   endp


;--------------------------------------------------------------
;is_it_driver - Given an address, search driver chain to see  |
;               if the address points to any of the drivers.  |
;                                                             |
;               This code could be enhanced for DOS versions  |
;               4.x or better, to look for drivers by         |
;               searching the subsegment control blocks of    |
;               MCBs for "D"river entries.                    |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                 drv_ptr = pointer to NUL device driver      |
;                           header                            |
;                   dx:bx = address to check                  |
;                                                             |
;              Exit:                                          |
;                                                             |
;                 Carry set if address is found in list       |
;                   dx:bx = ptr to driver header              |
;                                                             |
;                                                             |
;                                                             |
;  es, si, ax preserved.                                      |
;--------------------------------------------------------------
is_it_driver    proc    near
                push    ax
                push    si
                push    es
                xor     bx, bx                  ;offset zero in handler
                les     si, drv_ptr             ;ptr to NUL device header
is_it_dr100:
                mov     ax, es                  ;driver segment
                cmp     ax, dx                  ;segment the same?
                jne     is_it_dr500
                cmp     si, bx                  ;offset the same?
                jne     is_it_dr500             ;if not, try again 
                stc                             ;indicate found
                jmp     short is_it_dr999       ;and exit
is_it_dr500:
                cmp     es:[si].d_offset, 0ffffh  ;end of driver chain?
                je      is_it_dr999
                les     si, es:[si]
                jmp     short is_it_dr100
is_it_dr999:
                pop     es
                pop     si
                pop     ax
                ret
is_it_driver    endp        


;--------------------------------------------------------------
;get_stacks_seg - Determine the segment where the DOS STACKS  |
;                 code resides.  This will be used later      |
;                 for tracing back through the STACKS         |
;                 interrupt handlers to find the original     |
;                 ISRs.                                       |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                atclassf = TRUE if AT class machine          |
;                 dos_ver = DOS minor ver/major ver           |
;                                                             |
;              Exit:                                          |
;                                                             |
;         dos_handler_seg = segment for DOS STACKS.           |
;                                                             |
;  ds preserved.                                              |
;--------------------------------------------------------------
get_stacks_seg  proc    near
                push    ds
                xor     bx, bx
                mov     ds, bx                  ;vectors                
                assume  ds:nothing
;
;Use the segment found in the floppy handler as our default guess of 
;where the STACKS code is.
;
;
                mov     bx, ((pcflop + BEG_INTS) shl 2)  ;vector entry for IRQ6
                mov     ax, [bx].d_segment           

                cmp     byte ptr cs:dos_ver, 4  ;DOS 4 or better?
                jb      get_stacks800        
;
;If DOS 4 or better, memory control block, subsegment control blocks
;can be used to find STACKS. 
;
                mov     ah, DOS_LIST_LISTS
                int     21h
                mov     dx, es:[bx - 2]         ;segment of first MCB
                mov     ds, dx
                mov     cx, dx                  ;save it
                xor     bx, bx
                cmp     [bx].mcb_sig, 'M'       ;valid MCB?
                jne     get_stacks800           ;MCBs must be corrupted

                mov     di, [bx].mcb_size       ;get first size
                inc     di                      ;add paragraph for the MCB
                add     dx, di                  ;location after DOS data seg entry
                
                inc     cx                      ;get to first subsegment block
get_stacks300:
                cmp     cx, dx                  ;at end of subsegment info?
                jae     get_stacks800           ;if so, search done
                mov     ds, cx                  ;load the segment
                cmp     [bx].mcb_sig, 'S'       ;STACKS?
                je      get_stacks500
                mov     cx, [bx].mcb_owner   
                add     cx, [bx].mcb_size       ;advance to next block
                jmp     short get_stacks300

get_stacks500:
                mov     ax, [bx].mcb_owner
get_stacks800:
                mov     cs:dos_handler_seg,  ax    ;save it
get_stacks900:
                pop     ds
                assume  ds:code    
                ret
get_stacks_seg  endp



;--------------------------------------------------------------
;verify_enables - Ensure that IRQs, enabled in the PIC        |
;                 masks are really being used.  These         |
;                 may just be pointing to default handlers    |
;                 (in which case they are really available).  |
;                                                             |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                atclassf = TRUE if AT class machine          |
;            math_present = TRUE if have math coprocessor     |
;                pic_mask = PIC mask values                   |
;            ver_irq_mask = bit map of IRQs to be verified    |
;                                                             |
;              Exit:                                          |
;                                                             |
;                 pic_mask  updated                           |
;                                                             |
;  bp preserved.                                              |
;--------------------------------------------------------------
verify_enables  proc    near
                push    bp
                xor     cx, cx                  ;use as irq counter
                xor     bx, bx
                mov     es, bx                  ;vectors                

                mov     bp, ver_irq_mask


                mov     dx, BEG_INTS 
verify_en100:                
                shr     bp, 1                   ;is this one we should verify?
                jnc     verify_en500            ;if not, go try next one

verify_en150:
                mov     bx, cx                  ;get IRQ number
                add     bx, dx                  ;get int number
                shl     bx, 1
                shl     bx, 1

                cmp     cl, MATH_IRQ            ;math coprocessor?
                jne     verify_en200
                cmp     math_present, TRUE
                jne     verify_en300            ;if not present and "enabled"
                jmp     short verify_en500

verify_en200:
                call    is_it_rom
                jc      verify_en500            ;no carry if in "BIOS"
verify_en300:

                mov     ax, 1
                shl     ax, cl                  ;get bit for IRQ
                or      pic_mask, ax            ;indicate not active    

verify_en500:
                inc     cx
                cmp     cx, 8                   ;done them all on master
                jne     verify_en700
                cmp     atclassf, TRUE
                jne     verify_en999            ;if PC have checked all
                mov     dx, (BEG_SLAVE_INTS - 8   )
verify_en700:
                cmp     cx, 16                  ;done them all on master and slave?
                jne     verify_en100

verify_en999:
                pop     bp
                ret
verify_enables  endp


;--------------------------------------------------------------
;is_it_rom -  Determine if the address of IRS is in ROM space.|
;                                                             |
;             Note:  It's possible that the first portion     |
;                    of ROM space at 0f000h is actually       |
;                    being used as a mapping area for upper   |
;                    memory blocks.  Thus, this routine       |
;                    would think that TSRs or drivers loaded  |
;                    high in this area were actually default  |
;                    handlers.  The program can be run with   |
;                    the /D switch (disable verification to   |
;                    get around this).                        |
;                                                             |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                   es:bx = ptr to ISR                        |
;         dos_handler_seg = segment of the DOS handlers       |
;                                                             |
;              Exit:                                          |
;                                                             |
;                Carry set if NOT in the BIOS                 |
;                                                             |
;  es, bx preserved.                                          |
;--------------------------------------------------------------
is_it_rom       proc    near
                push    bx
                push    es

                cmp     es:[bx].d_segment, 0f000h       ;in the BIOS?
                je      is_it_999
;
;This is a hack.  A memory manager was intercepting interrupts
;and routing them to handlers which began with an int 3h instruction.  If
;the ISR starts with that instruction, consider it to be have been originally
;pointing to the BIOS. 
;
                push    bx
                push    es
                les     bx, es:[bx]
                cmp     byte ptr es:[bx], 0cch          ;ISR starts with int 3?
                pop     es
                pop     bx
                je      is_it_999
                

;See if pointing to DOS stack handlers.  If so, carry is not set and es:bx
;points to address of original ISR.
;
                call    ck_for_dos
                jc      is_it_999                

                mov     ax, es
                cmp     ax, 0f000h                       ;in the BIOS?
                je      is_it_999
                cmp     byte ptr es:[bx], 0cch          ;ISR starts with int 3?
                je      is_it_999
                stc
is_it_999:
                pop     es
                pop     bx
                ret
is_it_rom       endp


;--------------------------------------------------------------
;ck_for_dos - Given a ptr to an interrupt vector, determine if|
;             the handler is one of those inserted by DOS     |
;             to first check stack before dispatching to      |
;             original ISR.  If it is, attempt to get the     |
;             address of the original ISR.                    |
;                                                             |
;             Enter:                                          |
;                      ds = local data segment                |
;                   es:bx = ptr to vector                     |
;         dos_handler_seg = segment of the DOS handlers       |
;                                                             |
;              Exit:                                          |
;                                                             |
;                Carry set if no DOS handler present or       |
;                          can't find original ISR.           |
;                                                             |
;             else  es:bx = ptr to original ISR               |
;                                                             |
;  All other registers preserved.                             |
;--------------------------------------------------------------
ck_for_dos      proc    near
                push    ax
                push    cx
                push    di

                mov     ax, dos_handler_seg 
                cmp     es:[bx].d_segment, ax   ;same segment as DOS handlers?
                jne     ck_for_900
                les     di, es:[bx]             ;es:di point to handler
                mov     al, es:[di]             ;get byte of code                

;
;Several different flavors of these stack handlers are used in the various
;iterations of DOS. Look for the following cases:
;
;case 1:             jmp  label
;                    dd   &old_isr
;
;            label:  call handler   ;put offset of &ISR on stack
;                    dw  offset var ;and go get it and call it
;
;
;case 2:             call handler   ;put offset of &ISR on stack
;                    dd   &old_isr
;
;
;case 3:             jmp  label
;                    dd   &old_isr
;
;             label:  ...
;
;                    pushf
;                    cs:
;                    call far [xxxx] ;also this address may not follow the
;                                    ;jmp

;case 4:             push ax  
;                    ... 
;                    ...
;
;                    pushf
;                    cs:
;                    call far [xxxx]   ;xxxx is the offset of the variable   
;                                      ;that holds the &ISR  

                cmp     al, OP_JMP      ;is it a jmp
                jne     ck_for_200
                inc     di
                sub     ah, ah
                mov     al, es:[di]     ;get next type of code
                inc     di              ;past 2nd byte of rel jmp instruction
                add     di, ax          ;get to location of jmp

                mov     al, es:[di]     ;get byte of code there
ck_for_200:
                cmp     al, OP_CALL     ;is it a call
                jne     ck_for_300
                mov     di, es:[di + 3] ;get offset to the dword that holds &ISR
                les     bx, es:[di]     ;&ISR
                jmp     short ck_for_999 ;exit with carry clear

ck_for_300:                
                mov     cx, BYTES_TO_SEARCH
ck_for_400:                
                mov     al, OP_PUSHF
                repne   scasb           ;pushf instruction?
                jcxz    ck_for_900
                mov     al, OP_CS       ;CS override?
                scasb
                jnz     ck_for_400
                mov     ax, OP_CALL_MEM16
                scasw                   ;call?
                jnz     ck_for_400
                mov     di, es:[di]     ;get offset of var that holds &ISR
                les     bx, es:[di]     ;get &ISR

ck_for_800:
                jmp     short ck_for_999 ;exit with carry clear
ck_for_900:
                stc
ck_for_999:
                pop     di
                pop     cx
                pop     ax
                ret
ck_for_dos      endp


;--------------------------------------------------------------
;retrieve_path - Given a segment containing environment data, |
;                find the string which holds the full path    |
;                and filename of the program launched from    |
;                that copy of the environment.                |
;                                                             |
;             Enter:                                          |
;                      es = environment segment               |
;              Exit:                                          |
;                                                             |
;                Carry set if string not found, or data       |
;                          doesn't look right                 |
;                                                             |
;             else     si = start of path                     |
;                      bx = start of filename portion         |
;   di preserved.                                             |
;--------------------------------------------------------------
retrieve_path   proc    near        
                push    di
                xor     di, di
                mov     cx, 8000h               ;max environment length
                xor     ax, ax                  ;look for end of environment
retrieve_100:
                repne   scasb
                jcxz    retrieve_900
                cmp     byte ptr es:[di], 0     ;double null?
                jne     retrieve_100
                add     di, 3                   ;skip it and next word
                mov     si, di                  ;hold start of path
                repne   scasb                   ;look for end
                jcxz    retrieve_900
                sub     di, 5                   ;back "x.xxx", 0
                cmp     byte ptr es:[di], '.'   ;right before extension?
                jne     retrieve_900            ;if data not right
                mov     cx, 8                   ;max chars it should take to find
retrieve_700:

                cmp     byte ptr es:[di - 1], 0
                je      retrieve_800
                cmp     byte ptr es:[di - 1], '\'
                je      retrieve_800
                dec     di 
                loop    retrieve_700
                jmp     short retrieve_900
retrieve_800:
                mov     bx, di                  ;return starting position
                clc
                jmp     short retrieve_999

retrieve_900:
                stc
retrieve_999:
                pop     di
                ret
retrieve_path   endp        

ck_monitor      proc    near
                int     11h
                and     al, 30h                 ;check for monochrome
                cmp     al, 30h
                jnz     ck_mon100
                mov     monoflag, TRUE          ;note that monochrome
                mov     monoatrib, TRUE         ;and use B/W attributes
ck_mon100:
                ret
ck_monitor      endp




code            ends
                end


